
#include "xc.h"

#include <pic18f27k42.h>

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <limits.h>
#include <string.h>
#include "i2c1_master.h"
#include "i2c1_master_example.h"


#pragma warning disable 1498
#pragma warning disable 31
#pragma warning disable 228   // caractere francais accentu
#pragma warning disable 520  
#pragma warning disable 1510 
#pragma warning disable 1518 
#pragma warning disable 759
#pragma warning disable  2053
#pragma warning disable  2020
#pragma warning disable 359
#pragma warning disable 373


// CONFIG1L
#pragma config FEXTOSC = OFF    // External Oscillator Selection->Oscillator not enabled
#pragma config RSTOSC = HFINTOSC_64MHZ    // Reset Oscillator Selection->HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1
// CONFIG1H
#pragma config CLKOUTEN = OFF    // Clock out Enable bit->CLKOUT function is disabled
#pragma config PR1WAY = OFF    // PRLOCKED One-Way Set Enable bit->PRLOCK bit can be set and cleared repeatedly
#pragma config CSWEN = OFF    // Clock Switch Enable bit->The NOSC and NDIV bits cannot be changed by user software
#pragma config FCMEN = OFF    // Fail-Safe Clock Monitor Enable bit->Fail-Safe Clock Monitor disabled
// CONFIG2L
#pragma config MCLRE = EXTMCLR    // MCLR Enable bit->If LVP = 0, MCLR pin is MCLR; If LVP = 1, RE3 pin function is MCLR 
#pragma config PWRTS = PWRT_OFF    // Power-up timer selection bits->PWRT is disabled
#pragma config MVECEN = ON   // Multi-vector enable bit->Interrupt contoller does not use vector table to prioritze interrupts
#pragma config IVT1WAY = ON    // IVTLOCK bit One-way set enable bit->IVTLOCK bit can be cleared and set repeatedly
#pragma config LPBOREN = OFF    // Low Power BOR Enable bit->ULPBOR disabled
#pragma config BOREN = SBORDIS    // Brown-out Reset Enable bits->Brown-out Reset enabled , SBOREN bit is ignored
// CONFIG2H
#pragma config BORV = VBOR_2P45    // Brown-out Reset Voltage Selection bits->Brown-out Reset Voltage (VBOR) set to 2.45V
#pragma config ZCD = OFF    // ZCD Disable bit->ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
#pragma config PPS1WAY = OFF    // PPSLOCK bit One-Way Set Enable bit->PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence)
#pragma config STVREN = ON    // Stack Full/Underflow Reset Enable bit->Stack full/underflow will cause Reset
#pragma config DEBUG = OFF    // Debugger Enable bit->Background debugger disabled
#pragma config XINST = OFF    // Extended Instruction Set Enable bit->Extended Instruction Set and Indexed Addressing Mode disabled
// CONFIG3L
#pragma config WDTCPS = WDTCPS_13    // WDT Period selection bits->Divider ratio 1:262144
#pragma config WDTE = OFF  //was SWDTEN    // WDT operating mode->WDT enabled/disabled by SWDTEN bit
// CONFIG3H
#pragma config WDTCWS = WDTCWS_7    // WDT Window Select bits->window always open (100%); software control; keyed access not required
#pragma config WDTCCS = MFINTOSC    // WDT input clock selector->WDT reference clock is the 32kHz MFINTOSC output
// CONFIG4L
#pragma config BBSIZE = BBSIZE_512    // Boot Block Size selection bits->Boot Block size is 512 words
#pragma config BBEN = OFF    // Boot Block enable bit->Boot block disabled
#pragma config SAFEN = OFF    // Storage Area Flash enable bit->SAF disabled
#pragma config WRTAPP = OFF    // Application Block write protection bit->Application Block not write protected
// CONFIG4H
#pragma config WRTB = OFF    // Configuration Register Write Protection bit->Configuration registers (300000-30000Bh) not write-protected
#pragma config WRTC = OFF    // Boot Block Write Protection bit->Boot Block (000000-0007FFh) not write-protected
#pragma config WRTD = OFF    // Data EEPROM Write Protection bit->Data EEPROM not write-protected
#pragma config WRTSAF = OFF    // SAF Write protection bit->SAF not Write Protected
#pragma config LVP = OFF    // Low Voltage Programming Enable bit->HV on MCLR/VPP must be used for programming
// CONFIG5L
#pragma config CP = OFF    // PFM and Data EEPROM Code Protection bit->PFM and Data EEPROM code protection disabled

#define _XTAL_FREQ 64000000 

#define WRITE_FLASH_BLOCKSIZE    128
#define ERASE_FLASH_BLOCKSIZE    128
#define END_FLASH                0x020000


#ifndef Byte
#define Byte  unsigned char
#endif
#ifndef Word
#define Word  uint16_t
#endif


#define VERSION "2023-0410"
#define Pass_Over   // evite de reconfigurer le sondes si rechargement programme
// ID# stocks en Eeprom
// rajout reconnaissance des 3 ID# sondes
//#define VERSION "2023-0407"

#define  MCU " 18F27K42"
#define POWER_SUPPLY " 3,3V "
#define DIR " C:\\MPLABX_PrOJECTS"
#define COMPILO "XC8 V2.36 C99"
#define PACK "PIC18F-K_DFP 1.6.125"


#define CR 13
#define LF 10
#define TAB 9

#ifdef With_SQA_Analyser
#define SQA_Dir TRISAbits.TRISA3
#define SQA LATAbits.LA3
#endif

#define Led_Vert    PORTAbits.RA3   //	Led Vert : CONFORT (SW1   Confort)					
#define LED_Bleu    PORTAbits.RA4   //	Led Bleu : ECO (SW1   Eco)					
#define LED_Jaune   PORTAbits.RA5 	//	Led Jaune : CMP (Compensation TC Ressentie)					
#define LED_Rouge   PORTAbits.RA6 	//	LED_rge	Led Rouge : PRG (Programmation)					
#define Bruleur     PORTAbits.RA7 	// 	BRL_cde	Commande Relais Brleur					

#define SW0     PORTBbits.RB0	    // SW0_PRM Menu Paramtres				
#define SW1     PORTBbits.RB1	    // SW1 CFE Alterner Confort / Eco				
#define SW2     PORTBbits.RB2	     // SW2_POS Augmenter Consignes				
#define SW3     PORTBbits.RB3	     // SW3_NEG Diminuer Consignes				
#define SW4     PORTBbits.RB4	     // SW4_PRG Menu Prog Hebdo (?)				
#define SW5     PORTBbits.RB5	     // SW5_HGR Menu  Horloge Hebdo (?)				

#define Def_Pins_PORTA  0b00000000   // leds bruleur
#define Def_Pins_PORTB  0b11111111   // B.P.
#define Def_Pins_PORTC  0b10100111   // UART1   I2C




#define Directory "C:\\MPLABX_Projects\\_18F27K42_OWS_DS18B20_2023.X"
#define Project " 18F27K42_OWS_DS18B20_2023"
#define Source  " main.c "
#define Annexe  " XC8 V2.36 C99  Pack: PIC18F-K_DFP 1.6.125"
#define Others  " DS18B20_barker.h  OneWire_barker.h  LCD_4bits_I2C.h UART1_Functions.h"
#define Eeprom  " Eeprom_PIC.h"
#define Hardware " Base 18F27K42 o "
#define Config " Internal Fosc 64MHz"

#define With_Test_Presence_Devices 


  const  char Msg0[]=" PIC18F27K42 + OWS sensor DS18B20 sur  PORT RA0)\r\n";
  const  char Msg1[]=" Directory :"Directory"\r\n";
  const  char Msg2[]=" Project : "Project"\r\n";
  const  char Msg3[]=" Source : "Source"_"VERSION"\r\n";
  const  char Msg4[]=" Config "Config"\r\n";
  const  char Msg5[]=" Autres : "Others"\r\n";
  const  char Msg6[]=" Eeprom :"Eeprom"\r\n";
  const  char Msg7[]=" RA0=DS18B20,RA4=led,RA3=SQA,RC6=TX UART1\r\n";
  const  char Msg8[]=" RC3=SCL,RC4=SDA,BP...BP5 RB0..R5\r\n";
  const  char Msg9[]=" ";
  const  char * Messages[]={Msg0,Msg1,Msg2,Msg3,Msg4,Msg5,Msg6,Msg7,Msg8,Msg9};
  const  char Blancs[]="  ";
  

#include "DS18B20_barker.h"  
#include "LCD_4bits_I2C.h"
#include "Eeprom_PIC.h"
#include "UART1_Functions.h"

/*  
  
 //PCF8574 pour LCD 2x16 avec A0,A1,A2  au niveau logique 0
  #define LCD_ADDR 0x20  
 //#define LCD_ADDR 0x27  //PCF8574 pour gros LCD 4x20 
 // Hardware :
 // PCF8574 I2C LCD Port Definition
 // P7,P6,P5,P4 = 4 bits Data  -> LCD D7..D4
 // P0=RS ,  P1=RW  not used car RW=0V , P2=Enb  P3=Backlight (BL) not used
 #define LCD_BL 0x08      // P3
 #define LCD_EN 0x04      // P2
 #define LCD_RW 0x02      // P1
 #define LCD_RS 0x01      // P0

#define LCD_CLEAR 0x01
#define LCD_HOME  0x02
#define Display_ON 0x0C
#define Display_OFF 0x08
  
  
 // LCD Command
#define LCD_LINE1 0x80      // 0x00 + bit D7=1
#define LCD_LINE2 0xC0      // 0x40 + bit D7=
#define LCD_LINE3 0x94 // 0x14 + bit D7=1
#define LCD_LINE4 0xD4 // 0x54 + bit D7=1
   
volatile int NbCarPL=16;
const unsigned char LINE[5] ={0x00,0x80,0xC0,0x94,0xD4};
const char Efface[17]="                ";

const unsigned char LCD_Custom_Chars_Set1[64] =
{
//---------- set #1 ------------------
0x0C,0x12,0x12,0x0C,0x00,0x00,0x00,0x00,  //char #1   Degre
0x06,0x09,0x08,0x1E,0x1E,0x08,0x09,0x06,  //char #2   euro
0x04,0x0A,0x11,0x11,0x0A,0x0A,0x11,0x00,  // char #3  Ohm
0x04,0x0E,0x0E,0x0E,0x1F,0x00,0x04,0x00,   // char #4   bell
0x1C,0x14,0x1B,0x12,0x13,0x12,0x12,0x00,//char #5 initiales PF
0x01,0x03,0x07,0x0F,0x07,0x03,0x01,0x00,  //char #6  A gauche
0x00,0x11,0x0A,0x04,0x06,0x11,0x00,0x00,  //char #7 cancel
0x00,0x01,0x03,0x16,0x1C,0x10,0x00,0x00  //char #8  Checked
};

const unsigned char LCD_Custom_Chars_Set2[64] =
{
//-----Barre verticales   set #2   0  6 colonnes , mais seulement 5 maxi en 5x8 cars  -------
0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,
0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,
0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,
0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,
0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,
0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,
0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,
0b11111111,0b11111111,0b11111111,0b11111111,0b11111111,0b11111111,0b11111111,0b11111111
};

const unsigned char LCD_Custom_Chars_Set3[64] =
{
//------- Set #3  ----------------------
0,0,0,0,27,27,27,27,
27,27,27,27,0,0,0,0,
3,3,3,3,24,24,24,24,
24,24,24,24,3,3,3,3,
0,0,0,31,31,31,31,31,
0,0,31,31,31,31,31,31,
0,31,31,31,31,31,31,31,
31,31,31,31,31,31,31,31
};
 
const char DegreC[8] ={    // 	176
0b01110000,
0b01010000,
0b01110111,
0b00000000,
0b00000111,
0b00000100,
0b00000100,
0b00000111
};

const char Special_Name_1[8][7]=
{"DEGREE","EURO  ","OHM   ","PF    ","Droite","Gauche","CANCEL","Check."}; // pour caracteres Speciaux
const char Special_Name_2[8][7]=
{ "Zero  ","un    ","deux  ","trois ","quatre","cinq  ","six   ","Sept  "};// pour Barre Graphe Horizontale


void LCD_init(void);
void LCD_putcmd(unsigned char d,unsigned char cmdtype);
void LCD_putch(unsigned char d);
void LCD_puts(char *s);
void LCD_putsRom(const char *s);
void LCD_Write_CText(const char *txt) ;
void Erase_Line(int L);
void LCD_At(unsigned char Ligne,unsigned char  col);
void LCD_Write_Text_At(unsigned char Ligne,unsigned char col, char * t1);
void LCD_Write_CText_At(unsigned char Ligne,unsigned char col,const char * t1);
void LCD_Cmd(unsigned char d);
void LCD_Load_Custom_Chars(unsigned char Set,int bavard);
 
  */


//---------------------------------------------------
  
  
 
#define UART1_RX_BUFFER_SIZE 128  

volatile uint16_t  i;
uint16_t  j,k,l,m,n;
uint16_t  Rep;
unsigned char  c1,c2,c3,cx,cc;

volatile unsigned char Buffer1[UART1_RX_BUFFER_SIZE];
volatile unsigned int Index1,i1;
volatile unsigned char Flag_Buffer1=0;

unsigned char TEXTE [128];
unsigned char * txt;
unsigned char * p0;
const unsigned char * p1;
const unsigned char * p2;
unsigned char * px;
unsigned char CRam1[80];
unsigned char Tampon[32];

float F0,F1,F2;
volatile uint16_t Count0, Count1 ;
volatile uint16_t Cpt0, Cpt1;
volatile uint16_t Flag_Timer1=0;
volatile uint16_t Flag_Timer3=0;
volatile uint16_t Flag_Timer6=0;
volatile signed int  Cpt6;  
volatile signed int  Cpt3;  
bool Status_GIE_Bit;


//   LCD 
volatile unsigned char Addr;
//volatile unsigned char Adresse_7bits;
unsigned char I2_Adresses_Devices[8];
unsigned char tmp[4] ;
int Etat;

volatile struct Anybits {   // 8 flags
       unsigned  Tmr0_Elapsed:1;     // for timer0 16bits
       unsigned  Tmr1_Elapsed:1;     // for timer1 16bits
       unsigned  Tmr2_Elapsed:1;     // for timer2 8bits
       unsigned  Tmr3_Elapsed:1;     // for timer3 16bits
       unsigned  Tmr4_Elapsed:1;     // for timer4 8bits
       unsigned  Tmr6_Elapsed:1;     // for timer6 8bits
       unsigned  NCO_initialise:1;   // for NCO
       unsigned  libre:1;
 }Drapeaux ;
 
 unsigned char Flag_Eeprom=0; 
 
unsigned char Eeprom_Donnees[][17]={"1234567890123456","1234567890123456","1234567890123456","1234567890123456",\
"1234567890123456","1234567890123456","1234567890123456","1234567890123456","1234567890123456",\
"1234567890123456","1234567890123456","1234567890123456","1234567890123456","1234567890123456"\
 };


 
/* 
 #include "Eeprom_PIC.h"
#ifdef Pass_Over
// eeprom config  4 sondes DS18B20 
__EEPROM_DATA (99,0,0,0,0,0,0,0);   
__EEPROM_DATA (0,0,0,0,0,0,0,0); // 0x0000
__EEPROM_DATA (0x28,0xFF,0x47,0x15,0x81,0x16,0x03,0x24); //  #1  Data: FamilyCode 28 ID2= FF4715811603 CRC= 24
__EEPROM_DATA (0,0,0,0,0,0,0,0); // 0x0010
__EEPROM_DATA (0x28,0xAA,0x42,0x65,0x26,0x13,0x02,0xB6); //  #2 Data: FamilyCode 28 ID1= FF83C7641503 CRC= B8
__EEPROM_DATA (0,0,0,0,0,0,0,0); // 0x0020 
__EEPROM_DATA (0x28,0xAA,0x56,0xC1,0x38,0x14,0x01,0x40); //  #3
__EEPROM_DATA (0,0,0,0,0,0,0,0); // 0X0030
__EEPROM_DATA (0x28,0xAA,0x0D,0x82,0x38,0x14,0x01,0xCD); // repre #6 !  
__EEPROM_DATA (0,0,0,0,0,0,0,0); // 0x0040
#else
// eeprom config  4 sondes DS18B20 
__EEPROM_DATA (0,0,0,0,0,0,0,0);    __EEPROM_DATA(0,0,0,0,0,0,0,0); // 0x0000
__EEPROM_DATA (0,0,0,0,0,0,0,0);    __EEPROM_DATA(0,0,0,0,0,0,0,0); // 0x0010
__EEPROM_DATA (0,0,0,0,0,0,0,0);    __EEPROM_DATA(0,0,0,0,0,0,0,0); // 0x0020
__EEPROM_DATA (0,0,0,0,0,0,0,0);    __EEPROM_DATA(0,0,0,0,0,0,0,0); // 0X0030
__EEPROM_DATA (0,0,0,0,0,0,0,0);    __EEPROM_DATA(0,0,0,0,0,0,0,0); // 0x0040
#endif


 
void DATAEE_WriteByte(uint16_t bAdd, unsigned char bData);
unsigned char DATAEE_ReadByte(uint16_t bAdd);
void Sauve_Parametres_en_Eeprom(void);
void Relecture_Eeprom(char Visu);  // visu seulement, sans rechargement !
void Sauve_Parametres_en_Eeprom() ;   // en ascii !
unsigned char  Restitue_Parametres_en_Eeprom(int Element);
 void Relecture_ID_datas(char *Po);
*/

//  ************************************************************


const unsigned short TEMP_RESOLUTION = 12;
// choix sensor 1 ou 2
unsigned char Valid_Sensor=0x00;
char Degres;
char Signe;
//   OWS
unsigned int DS18B20_raw[3];
unsigned char OWS_address[8]= {0,0,0,0,0,0,0,0};
unsigned char temp1[8] = {1,2,3,4,5,6,7,8}; 
char *temp = "000.00 C";
unsigned char Valid_Sensor;
int temper;


/* Adresses des 3 capteurs de temprature */
unsigned char  SENSOR_ADDRESS[3][8]=
{
#ifdef Pass_Over     // snon sur rechargment programme ,il faut reconfigurer les sondes !
  {0x28,0xFF,0x47,0x15,0x81,0x16,0x03,0x24}, //  #1  Data: FamilyCode 28 ID2= FF4715811603 CRC= 24
  {0x28,0xAA,0x42,0x65,0x26,0x13,0x02,0xB6}, //  #2 Data: FamilyCode 28 ID1= FF83C7641503 CRC= B8
  {0x28,0xAA,0x56,0xC1,0x38,0x14,0x01,0x40} //  #3  Data: FamilyCode 28 ID2= FF4715811603 CRC= 24
#else
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, //  #1  Data: FamilyCode 28 ID2= FF4715811603 CRC= 24
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, //  #2 Data: FamilyCode 28 ID1= FF83C7641503 CRC= B8
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00} //  #3  Data: FamilyCode 28 ID2= FF4715811603 CRC= 24
#endif
  };

unsigned long  DS18B20_Temp1;
unsigned int temp_fraction;
char TH1=19;
char TL1=15;
unsigned long  DS18B20_Temp2;
char TH2=20;
char TL2=14;


// *****************************************************

void Interrupt_Init(void);
void Hardware_Init();
void TMR6_Init_500ms(char N);

//void Init_I2C (void);
void I2C1_Write1ByteRegister(i2c1_address_t address, unsigned char reg, unsigned char data);
void I2C_PCF8574_Write(unsigned char  Addr , unsigned char value);
//void Init_2_LCD(void);



//--------------------------------------------------------------------------------------------

char Convert_Temperature(unsigned int temp2write);
void Adresse_Device_DS18B20(int N);
// void Update_Thresholds(int numX);
void Lecture_ID_Sensor(int NS);
//------------------------


 
/*
 * #include "UART1_Functions.h"
void  UART1_Init(void); 
void RAZ_UART1_Buffer1(void);
void PrintChar(unsigned char txData);
void Print (char * T1);
void CPrint (const char * T1);
void CRLF1(void);
void strConstRamCpy(unsigned char  *dest, const  unsigned char *source);
void Decompile_byte(unsigned char un);
 
*/

void __delay_xS(unsigned char N);


//---------------------------------------------------------


void  Interrupt_Init(void)
{
    INTCON0bits.IPEN = 1;
    bool state = (unsigned char)  INTCON0bits.GIE;
   
    INTCON0bits.GIEH = 0; // dishigh priority interrupts
    INTCON0bits.GIEL = 0; // dis low priority interrupts
    INTCON0bits.IPEN = 1; // Enable interrupt priority
   //  __asm("BCF INTCON0,GIE");
   // GIE = 0;
    IVTLOCK = 0x55;
    IVTLOCK = 0xAA;
    IVTLOCKbits.IVTLOCKED = 0x00; // unlock IVT

    IVTBASEU = 0;
    IVTBASEH = 0;
    IVTBASEL = 8;

    IVTLOCK = 0x55;
    IVTLOCK = 0xAA;
    IVTLOCKbits.IVTLOCKED = 0x01; // lock IVT

    INTCON0bits.GIE= state;

    // Assign peripheral interrupt priority vectors
    IPR3bits.U1RXIP = 1;
    IPR1bits.INT0IP = 1;
    IPR4bits.TMR1IP = 1;
    IPR0bits.IOCIP=0;
    IPR3bits.TMR0IP = 1;
    
    
}

/*
void (*IOCAF0_InterruptHandler)(void);
void (*IOCAF1_InterruptHandler)(void);
void (*IOCAF2_InterruptHandler)(void);
void (*IOCAF3_InterruptHandler)(void);
void (*IOCAF4_InterruptHandler)(void);
void (*IOCAF5_InterruptHandler)(void);
void (*IOCAF6_InterruptHandler)(void);
*/

void __interrupt(irq(IRQ_U1RX),high_priority) UART1_rx_vect_isr(void)
{
   static char cx;
  //  U1TXB='H';  // pour debugging only
  if ( (PIR3bits.U1RXIF==1) && (PIE3bits.U1RXIE ==1) )
  {   
      c1=U1RXB;
   //   U1TXB='*'; // echo
     if ((c1==13) || (i1>UART1_RX_BUFFER_SIZE-1))
     { 
      Flag_Buffer1=1;
      Buffer1[i1]=0;
      Index1=i1;
      i1=0; 
     PIE3bits.U1RXIE =0;
     }
     else
     {
       Buffer1[i1]=c1;
       i1++;
       Index1=i1;
      } 
   } 
}


void __interrupt(irq(IRQ_TMR1),high_priority)TMR1_ISR(void)
{        
     if ( Cpt1 < 1)   // x 100ms at 500Khz
      {
      T1CONbits.TMR1ON=0;// stop comptage  timer1
      PIE4bits.TMR1IE=0;
      Drapeaux.Tmr1_Elapsed=1;
      }
      else
      {
       Cpt1--;
      }
      // 100mS  avec prescaler =1/2  et bascule  1/2
      TMR1H         = 0xCF;  // 53036
      TMR1L         = 0x2C;
       PIR4bits.TMR1IF=0;
  }

void __interrupt(irq(IRQ_TMR6),high_priority) TMR6_ISR(void)
{  
    if (  (TMR6IF==1) && (TMR6IE==1) )
    {
      if (Cpt6>0)  // decomptage maxi 32767
       {
         Cpt6--;
        }
      else
      {
        Drapeaux.Tmr6_Elapsed=1;
        TMR6IE=0;
      } 
     Flag_Timer6=1;
     TMR6IF=0;// clear the TMR6 interrupt flag
     
    }       
 }

//-------------  EEPROM  PIC ----------------------------------------
/*

void DATAEE_WriteByte(uint16_t bAdd, unsigned char bData)
{
    unsigned char GIEBitValue = INTCON0bits.GIE;

    NVMADRH = (unsigned char)((bAdd >> 8) & 0x03);
    NVMADRL = (unsigned char)(bAdd & 0xFF);
    NVMDAT = bData;
    NVMCON1bits.NVMREG = 0;
    NVMCON1bits.WREN = 1;
    INTCON0bits.GIE = 0;     // Disable interrupts
    NVMCON2 = 0x55;
    NVMCON2 = 0xAA;
    NVMCON1bits.WR = 1;
    // Wait for write to complete
    while (NVMCON1bits.WR)
    {
         __asm("NOP");
    }

    NVMCON1bits.WREN = 0;
    INTCON0bits.GIE = GIEBitValue;   // restore interrupt enable
}

unsigned char DATAEE_ReadByte(uint16_t bAdd)
{
    NVMADRH = (unsigned char)((bAdd >> 8) & 0x03);
    NVMADRL = (unsigned char)(bAdd & 0xFF);
    NVMCON1bits.NVMREG = 0;
    NVMCON1bits.RD = 1;
    __asm("NOP");  // NOPs may be required for latency at high frequencies
    __asm("NOP");

    return (NVMDAT);
}


 

void Relecture_Eeprom(char Visu)
{ // Visu seulement !
    static unsigned char cc;
    static int i,j,k;
    i=0; j=0;k=0;
    cc= DATAEE_ReadByte(0);
    Flag_Eeprom= cc;
    __delay_ms(25);
    if (Visu==1)
     {
        CPrint(" Relecture datas eeprom : \r\n");
        CPrint(" Flag @ 0x0000 =");
        PrintChar(cc/10+48);  PrintChar(cc%10+48); CRLF1();
     }
     for (i=0;i<3;i++)
     {
          if (Visu==1)
          {   
           CPrint(" Code Sonde #");PrintChar(i+48);  ;PrintChar(':'); 
          }
          k=(i*16)+0x10 ;
          if (Visu==1)
          {
            // utoa(CRam1,k,10);
             sprintf(CRam1," 0x%04X  ",k);
             Print(CRam1); 
             PrintChar(TAB);
          }
          for (j=0;j<8;j++)
          {
              cc= DATAEE_ReadByte(k+j); __delay_ms(25);
              Eeprom_Donnees[i][j]=cc;
             sprintf(CRam1,"0x%02X,",cc );	// en ascii	   
             Print(CRam1);
          }
           Eeprom_Donnees[i][j]=0;    //string terminator
          if(Visu==1)
          {
           //  PrintChar(TAB);
             //Print( Eeprom_Donnees[i]);
           //  sprintf(CRam1,"0x%02X",Eeprom_Donnees[i]);	// en ascii	   
           //  Print(CRam1);
             CRLF1();
           }
     }
     if(Visu==1)  CRLF1();
  }


void Sauve_Parametres_en_Eeprom()   
{    static unsigned char cc;
    static int i,j,k;
    i=0; j=0; k=0;
    GIEH=0;
    //Flag_Eeprom=99;
    CPrint("\r\n Sauve ID# sondes DS18B20 \r\n");
    CPrint(" Flag Eeprom @0x0000 = ");
    DATAEE_WriteByte(0,Flag_Eeprom); __delay_ms(25);
    PrintChar((Flag_Eeprom/10) +48);    PrintChar((Flag_Eeprom%10) +48);CRLF1();
  
    for (i=0;i<3;i++)
      {  CPrint(" Sonde ID#");PrintChar(i+48); 
         k=(i*16)+ 0x0010;
         sprintf(CRam1,"@ 0x%04X  -> ",k);
         Print(CRam1); 
         for (j=0;j<8;j++)
         {
           cc= SENSOR_ADDRESS[i][j];  
            DATAEE_WriteByte(k+j,cc); __delay_ms(25);
          sprintf(CRam1,"0x%02X,",cc);	// en ascii	   
          Print(CRam1);
          
         }
         CRLF1();
       }
     CRLF1();
     GIEH=1;

 }

unsigned char  Restitue_Parametres_en_Eeprom(int Element)
{   // Eeeprom -> Tables de Travail
   static int i,j,k;
   static unsigned char cx;
   j=0;i=0;k=0;
      CPrint(" Read flag @0x00 = ");
      Flag_Eeprom=DATAEE_ReadByte(0);__delay_ms(50);
      sprintf(CRam1,"0x%02X", Flag_Eeprom);
      Print(CRam1);
    // utoa(CRam1, Flag_Eeprom,10);
   //   My_Print(CRam1);CRLF1();
     CPrint(" Restitue sonde ID#");
     PrintChar(Element+48);CRLF1();
     if ( (Element>3) || (Element<1))  return 0; 
     k= (Element<<4) + 0x0010; // Element*16 +10
     sprintf(CRam1,"@ 0x%04X ", k);
     Print(CRam1);
     for (j=0;j<8;j++)
     {
        cx= DATAEE_ReadByte(k+j); __delay_ms(25);
       SENSOR_ADDRESS[Element-1][j]=cx;
       sprintf(CRam1,"0x%02X,",cx);
       Print(CRam1);
      }
      SENSOR_ADDRESS[Element-1][j]=0;  //string terminator
      CRLF1();
     //last check :
      Print((char *)&SENSOR_ADDRESS[Element-1][0]);
      CRLF1();
 }
       

*/

//-------------   OWS -------------------------------------------------------


char Convert_Temperature(unsigned int temp2write)
{ 
  const unsigned int RES_SHIFT = TEMP_RESOLUTION - 8;
  char i;
  char temp_whole;
  DS18B20_Temp1=0;
  // Check if temperature is negative
  if (temp2write & 0x8000) {
     Signe=1;
     temp2write = ~temp2write + 1;  // complement  2
     }
     else
     Signe=0;
  // Extract temp_whole
  temp_whole = temp2write >> RES_SHIFT ;
  if (Signe==1) temp_whole= -temp_whole;
  DS18B20_Temp1= ( long)temp_whole;
  // Extract temp_fraction and convert it to unsigned int
  temp_fraction  = temp2write << (4-RES_SHIFT);
  temp_fraction &= 0x000F;
  temp_fraction *= 625;
  DS18B20_Temp1= DS18B20_Temp1 *10000 + (unsigned long)temp_fraction;
  return(temp_whole);
}

void Adresse_Device_DS18B20(int N)
{
   int i;
    // Ow_Reset(&PORTA, 0);       // Onewire reset signal
     onewireInit();
     //The match ROM command followed by a 64-bit ROM code sequence allows
     //the bus master to address a specific slave device on a multidrop or single-drop bus.
     // Only the slave that exactly matches the 64-bit ROM code sequence will respond to
     // the function command issued by the master
     __delay_us(250);
     onewireWriteByte( THERM_CMD_MATCHROM); //0x55
     for (i=0;i<8;i++)
      {
       onewireWriteByte(SENSOR_ADDRESS[N-1][i])  ;
        __delay_us(250);
      }
     __delay_us(250);
 }

void Lecture_ID_Sensor(int NS)
{   unsigned char c1;
    unsigned int AC;  // Adresse courante eeprom
    char IDx[9];
     txt=&TEXTE[0];*(txt)=0; // init pointeur txt
     CPrint("\r\n 1 seul capteur DS18B20 doit etre connect\r\n");
     onewireInit();
     //THERM_CMD_READROM 0x33 Read ROM (if only one device)
     onewireWriteByte(0x33);
     __delay_us(750);
     c1=onewireReadByte();
     //  gabarit          0123456789012345678901234567890123456789012345678
     strConstRamCpy(txt ,"Data: FamilyCode .. ID.= xxxxxxxxxxxx CRC= ..");
     *(txt+22)=NS+48;
     //64-Bit Lasered ROM Code  8-BIT FAMILY CODE (28h)
    // 48-BIT SERIAL NUMBER contain a unique serial number.
     // 8-BIT CRC     
      sprintf(CRam1,"%02X",c1);
    // AC=0x0010 + NS*16 ;
      IDx[0]=c1;
   //  EEPROM_Write(AC,c1);
     SENSOR_ADDRESS[NS-1][0]=c1;
     __delay_ms(5);        // was 25
     memcpy(txt+17,CRam1,2);  // family type
     for (i=1;i<7;i++)
     {
     c1=onewireReadByte();
     IDx[i]=c1;
   //  EEPROM_Write(AC+i,c1);
     SENSOR_ADDRESS[NS-1][i]=c1;
     __delay_ms(5);    // was 25
     sprintf(CRam1,"%02X",c1);
     memcpy(txt+23+i*2,CRam1,2);
     }
     c1=onewireReadByte(); // CRC
   //  EEPROM_Write(AC+i,c1);
     SENSOR_ADDRESS[NS-1][7]=c1;
     sprintf(CRam1,"%02X",c1);
     IDx[7]=c1;
     memcpy(txt+43,CRam1,2);
     Print(txt);
     CRLF1() ;
     __delay_ms(25); // was 750
     Valid_Sensor=Valid_Sensor | NS;
}

//------------------------------------------------------
void __delay_xSec(unsigned char N)
{
static int i2;  
for (i2=0;i2<N;i2++)  __delay_ms(1000);
}

/*
void UART1_Init(void)
{
    U1P1L = 0x00;
    U1P1H = 0x00;
    U1P2L = 0x00;
    U1P2H = 0x00;
    U1P3L = 0x00;
    U1P3H = 0x00;
    U1CON0 = 0xB0; // BRGS high speed; MODE Asynchronous 8-bit mode; RXEN enabled; TXEN enabled; ABDEN disabled; 
    // RXBIMD Set RXBKIF on rising RX input; BRKOVR disabled; 
    // WUE disabled; SENDB disabled; ON enabled; 
    U1CON1 = 0x80;    
    // TXPOL not inverted; FLO off; C0EN Checksum Mode 0; RXPOL not inverted;
    //RUNOVF RX input shifter stops all activity; STP Transmit 1Stop bit, receiver verifies first Stop bit; 
    U1CON2 = 0x00;
    U1BRGL = 0x8A; // BRGL 138;   115200 bds
    U1BRGH = 0x00; // BRGH 0; 
    U1FIFO = 0x00; // STPMD in middle of first Stop bit; TXWRE No error; 
    // ABDIF Auto-baud not enabled or not complete; 
    // WUIF WUE not enabled by software; ABDIE disabled; 
    U1UIR = 0x00;
    // ABDOVF Not overflowed; TXCIF 0; RXBKIF No Break detected; 
    // RXFOIF not overflowed; CERIF No Checksum error; 
    U1ERRIR = 0x00;
    // TXCIE disabled; FERIE disabled; TXMTIE disabled; ABDOVE disabled;
    // CERIE disabled; RXFOIE disabled; PERIE disabled; RXBKIE disabled; 
    U1ERRIE = 0x00;
    U1CON0bits.TXEN=1;
    U1CON0bits.RXEN=1;  // enable recpetion
    PIE3bits.U1RXIE = 0;  // disable receive interrupt
}


void strConstRamCpy(unsigned char   *dest, const  unsigned char *source)
 { static unsigned char cc=0;
  
 cc= *(source);
  while ( cc>0 )
  { cc=*(source);
   *(dest++) =cc;
   source++ ;
  }
  *(dest) = 0 ;    // terminateur
}




void PrintChar( unsigned char txData)
{
    while(0 == PIR3bits.U1TXIF)
    {
        __asm("NOP");
    }
    U1TXB = txData;    // Write the data byte to the USART.
}

 void Print (char * Tc)
 {  int k;
   k=0;      
   while(*(Tc+k)>0) 
   {
     while(PIR3bits.U1TXIF==0);
     U1TXB=*(Tc+k);
    k++;
    if(k>128)break;
   }
}


  void CPrint (const char *T1)
 { 
   while(*(T1)>0) 
   {
     while(PIR3bits.U1TXIF==0);
     U1TXB=*(T1);
    T1++;
   }
}

void CRLF1(void)
{
   while(PIR3bits.U1TXIF==0);
   U1TXB=13;
    while(PIR3bits.U1TXIF==0);
   U1TXB=10;  
  }   


void RAZ_UART1_Buffer1(void)
{ int i1;   
      if (PIR3bits.U1RXIF==1)   c1=U1RXB ;
      for (i1=0;i1<sizeof(Buffer1)-1;i1++) Buffer1[i1]=0;
      Flag_Buffer1 =0 ;
      U1CON0bits.U1RXEN=1;
      U1ERRIR = 0x00;
      U1ERRIE = 0x00;
      Buffer1[0]=0;
      i1=0;
      Index1=0;
      c1=0;
      PIE3bits.U1IE  = 1;       // int uart1 enabled
      PIE3bits.U1RXIE =1;  
} 



void Decompile_byte(unsigned char un)
{
 unsigned char masque;
 masque = 0x80;
 while (masque > 0u )
 {
   if ((un & masque)>0)
      PrintChar('*');  //   UART1_Write(49u);  //  '1'
    else
       PrintChar(' ');  // UART1_Write(48u);  //  '0'
    masque =masque >>1;
  }
}
 
*/

void I2C_PCF8574_Write(unsigned char Adrp,unsigned char value)
{ 
 tmp[0]=value;
 tmp[1]=0;
 I2C1_WriteNBytes(Adrp, tmp, 1); 
  __delay_us(30);
} 


//----------------  Timers ---------------------

void TMR1_Initialize(void)
{
    //Set the Timer to the options selected in the GUI
    //T1GE disabled; T1GTM disabled; T1GPOL low; T1GGO done; T1GSPM disabled; 
    T1GCON = 0x00;
    T1GATE = 0x00;//GSS T1G_pin; 
    T1CLK = 0x05; //CS MFINTOSC_500kHz; 
    TMR1H = 0x0B;  //TMR1H 11;
    TMR1L = 0xDC; //TMR1L 220; 
    // Clearing IF flag before enabling the interrupt.
    PIR4bits.TMR1IF = 0;
    // Enabling TMR1 interrupt.
    PIE4bits.TMR1IE = 1;
    // CKPS 1:4; NOT_SYNC synchronize; TMR1ON enabled; T1RD16 disabled; 
    T1CON = 0x21;
    // Start the Timer by writing to TMRxON bit
    //T1CONbits.TMR1ON = 1;
}


void TMR6_Init_500ms(char N)
{   
    //  cycle=500 mS modifi -> 1sec !
     PIE9bits.TMR6IE=0 ;
     // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T6CLKCON = 0x05;    // T2CS FOSC/4 ou    0x05 pour 500Khz
    T6HLT = 0x00;
    T6TMR = 0x00;  // TMR4 0; 
    T6CON = 0x70 | 0x07 ; // Prescaler =7=> 1/128 Poscaler=7=>  1/8 =>  ticks= 512  S
    T6PR=244;   // 512 * (244+1) * 128 * 8 => 501760S ->  500mS sec
   Flag_Timer6=0;
   Drapeaux.Tmr6_Elapsed=0;
   TMR6ON = 1;
   TMR6IF=0;
   if(N>0) 
   { 
     Cpt6=N;
      PIE9bits.TMR6IE=1 ;  // TMR6IE_bit=1;
   }
   else
   {
       PIE9bits.TMR6IE=0 ; // TMR6IE_bit=0;
        Cpt6=0;      
   } 
 TMR6IF=0;   
 }

// -------------------------------------------------

void Hardware_Init(void)
{
    LATA = 0x00;
    LATB = 0x00;
    LATC = 0x00;

    TRISA = Def_Pins_PORTA;  //0x00
    TRISB = Def_Pins_PORTB; // 0xFF;
    TRISC = Def_Pins_PORTC; // 0x80; 
             

    ANSELC = 0x00;
    ANSELB = 0x00;
    ANSELA = 0x00;

    WPUE = 0x00;
    WPUB = 0x7F;
    WPUA = 0xFF;
    WPUC = 0xC0;  // RX TX UART

   // RxyI2C registers
    RB1I2C = 0x00;
    RB2I2C = 0x00;
   // RC3I2C = 0x00;
   // RC4I2C = 0x00;

   // ODx registers
    ODCONA = 0x07;  //  0x07 => open drain sur RA0,RA1,RA2  OWS sensor
    ODCONB = 0x00;
    ODCONC = 0x18;  // I2C 

   
  //  SLRCONx registers
    SLRCONA = 0xFF;
    SLRCONB = 0xFF;
    SLRCONC = 0xFF;

   
   // INLVLx registers
    INLVLA = 0xFF;
    INLVLB = 0xFF;
    INLVLC = 0xFF;
    INLVLE = 0x08;

    //IOCx registers 
        //interrupt on change PORT A   IOCAF - flag
    IOCAF=0;
    IOCAN=0;
    IOCAP=0;
           
	// I2C1
    I2C1SDAPPS = 0x14;  //RC4->I2C1:SDA1;    
    RC3PPS = 0x21;      //RC3->I2C1:SCL1;    
    RC4PPS = 0x22;      //RC4->I2C1:SDA1; 
    // UART1
    RC6PPS = 0x13;      //RC6->UART1:TX1;    
    U1RXPPS = 0x17;     //RC7->UART1:RX1;    
    I2C1SCLPPS = 0x13;  //RC3->I2C1:SCL1;    
}

// -------------  I2C --------------------------------

void Init_I2C (void)
{   
    I2C1CLK    = 0x03;      // I2C1 Clock Selection Register, MFINTOSC 500kHz ?? 100kHz ??  // doc p. 581
  // 2-04-2023  probleme maxi pCF SLC =100KHz
  //  I2C1CLK    = 0x06;    ///0110	 TMR2 post scaled output	42Khz theorique
  // SQA mesure  18.00+11.76=29.76S -> 33.6Khz ?? why ?
    I2C1CON0   = 0x04;      // I2C1 Control Register 0, Master mode, 7-bit Address  // doc p.577
    I2C1CON1   = 0x80;      // I2C1 Control Register 1, Acknowledge value transmitted after received data 
                            //(when I2CCNT=0) : 1 = NACK     (Not ACKnowledge)  // doc p.579
    I2C1CON2   = 0x00;      // I2C1 Control Register 2, envoi Start non auto, SDAHT 300ns hold time, BFRET 8 i2c clock pulses  // doc p.580
    I2C1CNT = 0;
 //   I2C1CON2bits.FME=1;
    I2C1CON2bits.ABD=0; //  ADB=0; // Transmitted Address data is loaded from the I2CxADB0/1 registers.
     // I2C1ADB1   = 0x3C;      //0x3C adresse bus I2C mini oled SSD1306
    I2C1ADB1   = 0x20 ;   //LCD_ADDR;   // LCD 2x16 avec PCF8754
    I2C1PIRbits.PCIF = 0;  // clear STOP bit flag
  //  I2C1CON2bits.ABD=1;   
    I2C1CON0bits.EN=0;   // i2c1 OFF   
}


void Init_Timer2()
{ 
  T2CON    = 0x00;
 // T2CONbits.CKPS=0b000;
   T2CONbits.CKPS2=0;// div 1/1 
    T2CONbits.CKPS1=0;
     T2CONbits.CKPS0=0;
   // T2CONbits.CKPS=0;     
  T2CONbits.OUTPS=1;  // div 1/2
  T2PR=190;  // 0.0625 * 1 * 2 * 190 =23.75 S  =>  42Khz
  T2TMR=0;
  T2CLK=0x02; //1= FOSC/4    2=FOSC     5=500Khz   
 // REGISTER 22-2: TxRST: TIMER2 EXTERNAL RESET SIGNAL SELECTION REGISTER p337
 // T2RST=0x04;  // CCP1OUT
    T2RST=0x00;  
  PIE4bits.TMR2IE=0;
  PIR4bits.TMR2IF=0;
  T2CONbits.T2ON=1;
}


 bool I2C1_DevicePresent(int16_t Address)
{
I2C1PIRbits.PCIF = 0;                // clear STOP bit flag
I2C1ADB1 = Address;                  // load slave Addr and R/W
I2C1CNT = 0;
I2C1CON0bits.S = 1;                  // send START + Addr
while (I2C1PIRbits.PCIF == 0);       // wait for STOP
return !I2C1CON1bits.ACKSTAT;
}

void Test_Presence_I2C_devices()
{
 int i,j;
 txt=&TEXTE[0]; 
 //*(txt)=0;
// I2C1CON2 = 0x00;  // <- OK
I2C1CON2bits.ABD=0; //  ADB=0; // Transmitted Address data is loaded from the I2CxADB0/1 registers.
I2C1CON0bits.EN = 1;
for (i=2; i<252; i=i+2)
{
if (I2C1_DevicePresent(i))
{
k=i>>1;
sprintf(txt," @ decimal #%4u  soit @Device 7bits = 0X%02X ",i,k);
//Print(txt);
switch (k)
{
case 0x20:// de base + 7 autres
case 0x21:
case 0x22:
case 0x23:
    //...etc 
case 0x27:
   strcat(txt," PCF8754 for LCD 2x16cars"); 
   break;
case 0x28:
case 0x29:
case 0x2A:
// case etc ..
strConstRamCpy(CRam1,"PCF8574 8bits IO" );  
j=strlen(txt);
strcpy(txt+j,CRam1);
//txt=strcat(txt,CRam1);// de base + 7 autres
break;
case 0x37:
     strcat(txt,"LCD 2119 2x16cars"); // de base + 7 autres
break;
case 0x38:
case 0x39:
case 0x3A:
case 0x3B:
// case etc ..
strcat(txt,"PCF8574A 8bits I/O"); // de base + 7 autres
break;
case 0x3C:
strcat(txt,"OLED SSD1306");
break;
case 0x57:
strcat(txt,"EEPROM 512bytes");
break;
case 0x68:
strcat(txt,"RTC DS3231");
break;
default:
strcat(txt,"non recens");
break;
}
Print(txt);CRLF1();
}
}
I2C1CON0bits.EN = 0;
//I2C1CON2 = 0x18;
I2C1CON2bits.ABD=1;
CRLF1();
}

//-------------  LCD ------------------------------------------------ 

/*

void Erase_Line(int L)
{
switch(L)
 {
 case 1:
  {
   LCD_Cmd(LCD_LINE1);   //LCD_putcmd(LCD_LINE1,1);   
    LCD_putsRom(Efface);      
   LCD_Cmd(LCD_LINE1); //LCD_putcmd(LCD_LINE1,1);            
   break;
  }
  case 2:
  {
   LCD_Cmd(LCD_LINE2); //LCD_putcmd(LCD_LINE2,1);   
   LCD_putsRom(Efface);     
   LCD_Cmd(LCD_LINE2);//LCD_putcmd(LCD_LINE2,1); 
    break;
  } 
  case 3:
  {
   LCD_Cmd(LCD_LINE3); //LCD_putcmd(LCD_LINE2,1);   
   LCD_putsRom("                    ");
   LCD_Cmd(LCD_LINE3);//LCD_putcmd(LCD_LINE2,1); 
    break;
  } 

  case 4:
  {
   LCD_Cmd(LCD_LINE4); //LCD_putcmd(LCD_LINE2,1);   
    LCD_putsRom("                    ");   
   LCD_Cmd(LCD_LINE4);//LCD_putcmd(LCD_LINE2,1); 
    break;
  } 

  }
 __delay_ms(10);
}

 //  LCD 2x16 + PCF8753
 // PCA8574 I2C LCD Routine
 // LCD Data PCA8574: P0,P1,P2,P3     Poids faibles !
 // LCD Control: P7: Back Light not used fix avec R pull up., P4=E-Enable, P5=not Used RW au Grnd , P6= RS



void LCD_putcmd(unsigned char d,unsigned char cmdtype)
{
  unsigned char lcddata;
 // avec RS=0
    // Put the Upper 4 bits data  on P7..P4
  lcddata =  (d & 0xF0) | LCD_EN | LCD_BL ;
   I2C_PCF8574_Write(Addr,lcddata );
  __delay_us(15);
  // Write Enable Pulse E: Hi -> Lo
   lcddata =  (d & 0xF0)| LCD_BL  ;
   I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // cmdtype = 0; One cycle write, cmdtype = 1; Two cycle writes
  if (cmdtype==1)
  {
    // Put the Lower 4 bits data   on P7 ..P4
    lcddata = ((d & 0x0F) <<4) | LCD_EN | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);    // __delay 2us for 16 MHz Internal Clock
    // Write Enable Pulse E: Hi -> Lo
     lcddata = ((d & 0x0F)<<4) | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);    // __delay 1us for 16 MHz Internal Clock
  }
}



void LCD_putch(unsigned char d)
{
  unsigned char lcddata;
   // Put the Upper 4 bits data     on P0 ..P3
  // avec RS=1
  lcddata =( (d & 0xF0) >>4 )| LCD_RS;
  I2C_PCF8574_Write(Addr,lcddata | LCD_EN);
  __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
  lcddata = (  ( (d & 0xF0) >>4 ) |  LCD_RS ) & (~LCD_EN) ;
  I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // Put the Lower 4 bit data
  lcddata = (d & 0x0F) | LCD_RS | LCD_EN;
  I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
  lcddata = ( (d & 0x0F) | LCD_RS ) & (~LCD_EN);
  I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
}


void LCD_Cmd(unsigned char d)
{
  unsigned char lcdcmd;
 // avec RS=0
    // Put the Upper 4 bits data  on P7..P4
    lcdcmd =   (d & 0xF0)  | LCD_EN | LCD_BL;
    I2C_PCF8574_Write(Addr,lcdcmd );
  __delay_us(15);
  // Write Enable Pulse E: Hi -> Lo
  // lcdcmd =  ( (d & 0xF0) >> 4  | LCD_RW  )  & (~ LCD_EN);
     lcdcmd =   (d & 0xF0) | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcdcmd);
    __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // Put the Lower 4 bits data   on P7....P4
     lcdcmd = ((d & 0x0F)<<4)| LCD_EN | LCD_BL;
     I2C_PCF8574_Write(Addr,lcdcmd);
      __delay_us(15);    // __delay 2us for 16 MHz Internal Clock
    // Write Enable Pulse E: Hi -> Lo
     lcdcmd = (d & 0x0F)<<4 | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcdcmd);
      __delay_us(15);    // __delay 1us for 16 MHz Internal Clock
}


void LCD_Data(unsigned char d)
{
  static  unsigned char lcddata;
   // Put the Upper 4 bits data     on P0 ..P3
  // avec RS=1
  lcddata=  ((d & 0xF0) )| LCD_RS | LCD_EN | LCD_BL;
  I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
 // lcddata =  ( ( (d & 0xF0) >>4 ) |  LCD_RS | LCD_RW  )  & (~LCD_EN);
  lcddata =   (d & 0xF0)  | LCD_RS | LCD_BL ;
  I2C_PCF8574_Write(Addr,lcddata);
   __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // Put the Lower 4 bit data
 // lcddata = (d & 0x0F) | LCD_RS | LCD_EN | LCD_RW;
  lcddata =  ( (d & 0x0F) <<4 ) | LCD_RS | LCD_EN | LCD_BL;
  I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
   lcddata =  ( (d & 0x0F) <<4 ) | LCD_RS | LCD_BL ;
  I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);     // __delay 1us for 16 MHz Internal Clock
}



void LCD_init(void)  // OK !
{
  
  // Wait for more than 15 ms after VCC rises to 4.5 V
  CPrint(" .1");
  LCD_putcmd(0x30,0);
  // Wait for more than 4.1 ms
  __delay_ms(5);
   PrintChar('2');
  // Send Command 0x30
  LCD_putcmd(0x30,0);
  __delay_ms(1);
  // Send Command 0x30
     PrintChar('3');
  LCD_putcmd(0x30,0);
    __delay_ms(1);
  // Function set: Set interface to be 4 bits long (only 1 cycle write).
      PrintChar('4');
 LCD_putcmd(0x25,0); 
    __delay_ms(1);
  // Function set: DL=0;Interface is 4 bits, N=1; 2 Lines, F=0; 5x8 dots font)
       PrintChar('5');
  LCD_putcmd(0x28,1);
     __delay_ms(1);
  // Display Off: D=0; Display off, C=0; Cursor Off, B=0; Blinking Off
       PrintChar('6');
  LCD_putcmd(0x08,1);
    __delay_ms(1);
  // Display Clear
       PrintChar('7');
  LCD_putcmd(LCD_CLEAR,1);
    __delay_ms(25);
  // Entry Mode Set: I/D=1; Increament, S=0; No shift
       PrintChar('8');
  LCD_putcmd(0x06,1);
      __delay_ms(1);
  // Display On, Cursor Off
       PrintChar('9');
  LCD_putcmd(0x0C,1);
    __delay_ms(1);
        PrintChar('A');
}



void LCD_putsRom(const char *s1)
{  uint16_t i;
    i=0;
   while((*s1 != 0)&& (i<NbCarPL))
  {
    LCD_Data(*s1); //     LCD_putch(*s1);
    s1++;i++;
     __delay_ms(1);
  }
}

void LCD_puts( char *s)
{  int i;
   i=0;
  while((*(s+i) != 0) && (i<NbCarPL))
  {
    LCD_Data(*(s+i));  // LCD_putch(*(s+i));
    i++;
  }
}
  

void LCD_Write_CText(const char *txt)
{  static int i1;
   i1=0;
   while ((*txt) && (i1<NbCarPL))
   i1++;
   LCD_Data(*(txt++)); //LCD_putch(*(txt++));
    __delay_ms(1);
 }


void LCD_Chr_At(int Ligne,int col,char c1)
{
  // Addr=LCD_ADDR;
  if(col>NbCarPL) col=NbCarPL;
  if (Ligne==1) LCD_Cmd(LCD_LINE1+col-1);  
  if (Ligne==2) LCD_Cmd(LCD_LINE2+col-1); 
   __delay_ms(25);
    LCD_Data(c1); //  LCD_putch(c1);
   __delay_ms(25);
 }


void LCD_At(unsigned char Ligne,unsigned char col)
{
  //Addr=LCD_ADDR;
  if(col>NbCarPL) col=NbCarPL-1;
  if (Ligne==1) LCD_Cmd(LCD_LINE1+col-1);  
  if (Ligne==2) LCD_Cmd(LCD_LINE2+col-1);  
  __delay_ms(20);
}

void LCD_Write_Text_At(unsigned char Ligne,unsigned char Col, char * t1)
{  LCD_At(Ligne,Col);
   LCD_puts(t1);
}


void LCD_Write_CText_At(unsigned char Ligne,unsigned char col, const char * t1)
{ char Table1[17];
 int k;
  k=strlen(t1);
  strConstRamCpy(Table1,t1);
  *(Table1+16)=0;
  if(col>=NbCarPL-k) col=1; 
  if (Ligne==1) LCD_Cmd(LCD_LINE1+col-1);
  if (Ligne==2) LCD_Cmd(LCD_LINE2+col-1);
  __delay_ms(20);
  LCD_puts(Table1);
}


//   affiche les pixels de chaque jeux de caracteres sur le terminal UART
void LCD_Load_Custom_Chars(unsigned char Set, int Cause)
{
    
 unsigned int  i, j;
 unsigned int  k;
 // Set address counter pointing to CGRAM address 0.
 LCD_Cmd(0x40);
  __delay_ms(150);
 // Load custom lcd character data into CGRAM.
 // It can only hold a maximum of 8 custom characters.
 k=0;
 if (Set==1)  CPrint(Special_Name_1[k]);
 if (Set==2)   CPrint(Special_Name_2[k]);
  if (Set==3)   CPrint(Special_Name_2[k]);// 2 car meme nomage
 CRLF1();
 k=1;
 for(i = 0; i <64; i++)
  {
    if ( (i>0) & (( i%8) ==0))
    {
    if (Set==1)  CPrint(Special_Name_1[k]);
    if (Set==2)   CPrint(Special_Name_2[k]);
    if (Set==3)   CPrint(Special_Name_2[k]); // meme nomage !
     k++;
     CRLF1();
    }
    switch (Set)
    {
        case 1:
         LCD_Data(LCD_Custom_Chars_Set1[i]);
         if (Cause==1)  Decompile_byte( LCD_Custom_Chars_Set1[i]);
            break;
       case 2:
         LCD_Data(LCD_Custom_Chars_Set2[i]);
         if (Cause==1)Decompile_byte(  LCD_Custom_Chars_Set2[i]);
            break;
       case 3:
         LCD_Data(LCD_Custom_Chars_Set3[i]);
         if (Cause==1)Decompile_byte(  LCD_Custom_Chars_Set3[i]);
            break;
      default:
          LCD_Data(LCD_Custom_Chars_Set1[i]);
         if (Cause==1)Decompile_byte( LCD_Custom_Chars_Set1[i]); 
         break; 
    }
    __delay_ms(25);
   if (Cause==1) CRLF1();
  }
 // Set address counter pointing back to the DDRAM.
 LCD_Cmd(0x80);
  __delay_ms(150);
}
*/

/*
  float GetTemper(unsigned char* address) 
  {
  //get temperature from the device with address address
  unsigned char i;
  float temperature;
  unsigned char scratchPad[9] = {0,0,0,0,0,0,0,0,0};
  onewireInit();
  onewireWriteByte(0x55);
  for (i = 0; i < 8; i++)  onewireWriteByte(address[i]);
  onewireWriteByte(0xBE);
  for (i = 0; i < 2; i++)  scratchPad[i] = onewireReadByte();
  onewireInit();
  temperature = ((scratchPad[1] * 256) + scratchPad[0])*0.0625;
  return temperature;
}
 * */
//---------------------------------------------------


void main(void)
{

    OSCCON1 = 0x60;
    OSCCON3 = 0x00;
    OSCEN = 0x00;
    OSCFRQ = 0x08;  // 64Mhz
    OSCTUNE = 0x00;
    __delay_ms(1500);
    Hardware_Init();
#ifdef With_SQA_Analyser
    SQA_Dir=0;
    SQA=0;
#endif
 
    for (i=0;i<sizeof(TEXTE);i++) TEXTE[i]=0;
    for (i=0;i<sizeof(Buffer1);i++) Buffer1[i]=0;
    for (i=0;i<sizeof(CRam1);i++) CRam1[i]=0;
    txt=&TEXTE[0]; 
    
    Valid_Sensor=0;
    #ifdef Pass_Over 
    Flag_Eeprom=99;
    #else
     Flag_Eeprom=0;
    #endif 
      
    UART1_Init();
    __delay_ms(500);
    CRLF1();
    CPrint(" Presentation :");
    for (i=0;i<9;i++)
    {
     CPrint(Messages[i]);   
    }
    CRLF1();
    k=sprintf(txt," Compile le %s a %s UTC\r\n avec version XC8 :  %d et PACK %s\r\n", __DATE__, __TIME__,__XC8_VERSION,PACK);
    Print(txt);
    CRLF1();
    
     CPrint(" Init_I2C();\r\n");
     Init_I2C(); // 500Khz / 5 = 100Khz

    
#ifdef With_Test_Presence_Devices
    CPrint(" Test presence devices sur Bus I2C1\r\n");
    Test_Presence_I2C_devices();
    CRLF1();
#endif    
    
    Addr=LCD_ADDR; 
    NbCarPL=16;
    sprintf(CRam1," adresse LCD= %d soit 0X%02X \r\n",Addr,Addr);
    Print(CRam1);
    I2C1CON2bits.ABD=1;
    
    CPrint(" \r\nSequence d'Init LCD 2x16 via I2C1 PCF8574\r\n");
    I2C1CON0bits.EN=1;   
    LCD_init(); 
    CRLF1();
    LCD_Cmd(LCD_CLEAR);// 0x01
     __delay_xSec(1);
    LCD_Cmd(LCD_HOME); //  0x02
    CPrint(" Affichage sur LCD 2x16 \r\n");
    
    LCD_putsRom("ABCDEFGHIJKLMNOP");
    Erase_Line(2);  __delay_ms(100);
    LCD_putsRom("1234567890123456");
    __delay_xSec(1);
    
     i=0;
    CPrint(" Chargement caracteres Speciaux set #1\r\n");
    LCD_Load_Custom_Chars(1,0);  // Set#1  mode silence
    CRLF1();
    
    
    Flag_Eeprom=    DATAEE_ReadByte(0);
    CPrint(" \r\n Config et Init One Wire\r\n");
    onewireInit();
    
       
    //==================== test 1 seule sonde ==========================
    
  //  broadcastConvert() ;
  //  CPrint("  Sensor Type, ID# , CRC  :\r\n");
  //  printSingleAddress(); // exemple: OWS ID# = {0x28,0xFF,0x47,0x15,0x81,0x16,0x03,0x24}
  //  CRLF1();
       // la 1ere sonde est forcment valide par defaut ! 
 //   memcpy(SENSOR_ADDRESS[0],OWS_address,8);
 //   Flag_Eeprom = 9;
 //   Sauve_Parametres_en_Eeprom();
    
#ifdef Pass_Over
    CPrint("  ATTENTION : Forage init EEPROM par programme  pendant le developpement Application,\r\n\
     pour eviter  une resaisie systematique des ID#  chaque rechargement de programme\r\n");
#endif
    CPrint("  Relecture et visu contenu Eeprom\r\n"); 
    Relecture_Eeprom(1); // parametre 1=> avec visu du contenu sur le terminal YAT UART1 
    CRLF1();
    __delay_xSec(2);
     if (Flag_Eeprom ==99)
    {    
      CPrint(" Parametres en Eeprom restitus OK et transfrs  -> table de travail  SENSOR_ADDRESS[[] \r\n"); 
      Erase_Line(1); __delay_ms(100);
     LCD_putsRom("Restitue Eeprom");
       Erase_Line(2); __delay_ms(100);
     LCD_putsRom("->3xID DS18B20 ");
     __delay_xSec(2);
     goto Suite ;
     }
     
    // on a pas le choix !
    CPrint(" ATTENTION\r\n  Saisie obligatoire des 3  ID# sondes DS18B20\r\n");  
    // --- Reconnaissance de l' ID# pour chacune des 3 sondes DS18B20    
    CPrint(" Test prsence d'un seul capteur O.W.S.  la fois sur RA0\r\n");
    CPrint (" Connecter la SONDE #1, puis appui sur SW0\r\n");
     Erase_Line(1); __delay_ms(100);
    LCD_putsRom("Connect SONDE #1");
     Erase_Line(2); __delay_ms(100);
    LCD_putsRom("et appui sur SW0");
    while(SW0==1)
      {};
    __delay_ms(20);
    while(SW0==0)
      {};
       
     broadcastConvert() ;
     CPrint("  Sensor Type, ID# , CRC  :\r\n");
     printSingleAddress();
     Erase_Line(1); __delay_ms(100);
     LCD_putsRom("ID#1 sensor     ");
     Erase_Line(2); __delay_ms(100);
     sprintf(CRam1,"%8c",OWS_address);
     LCD_Write_Text_At(2,1,CRam1);
     __delay_xSec(3);
     
    // la 1ere sonde est forcment valide par defaut ! 
    memcpy(SENSOR_ADDRESS[0],OWS_address,8);
    Flag_Eeprom=9;        // 9 pour ID#1   90 pour ID#1 et 2   99 pour les 3 ID             
    Sauve_Parametres_en_Eeprom() ;
    // verification visuelle
    Relecture_Eeprom(1);
    
  
    
Encore:   
 
    CPrint (" Connecter la SONDE 2, seule ,puis appui sur SW0\r\n");
     Erase_Line(1); __delay_ms(100);
    LCD_putsRom("Connect SONDE #2");
     Erase_Line(2); __delay_ms(100);
    LCD_putsRom("et appui sur SW0");
    __delay_ms(100);
    while(SW0==1)
    {}; 
     __delay_ms(20);
    while(SW0==0)
    {};
    
    LCD_Cmd(LCD_CLEAR);
    LCD_Cmd(LCD_HOME); //  0x02
    CPrint (" Recup ID# sonde #2 \r\n");
    
    broadcastConvert() ;
    CPrint("  Sensor Type, ID#2 , CRC  :\r\n");
    printSingleAddress();
    
    
     Erase_Line(1); __delay_ms(100);
     LCD_putsRom("ID#2 sensor     ");
     Erase_Line(2); __delay_ms(100);
     sprintf(CRam1,"%8c",OWS_address);
     LCD_Write_Text_At(2,1,CRam1);
     __delay_xSec(3);
    
    CPrint (" appui sur SW0 pour recommencer ou SW4 pour valider\r\n");
    Erase_Line(1);
    LCD_putsRom("valid par   SW4 ");
    Erase_Line(2);
    LCD_putsRom("annule par SW0  ");
      __delay_ms(100);
     while((SW4==1) && (SW0==1))
     __delay_ms(20);
    if ((SW4==1) && (SW0==0)) 
        goto Encore;
       
    memcpy(SENSOR_ADDRESS[1],OWS_address,8);
    Flag_Eeprom=Flag_Eeprom+81;   // 9+81=90
    Sauve_Parametres_en_Eeprom() ;
    Relecture_Eeprom(1);
     
   
Encore1:   
          
     CPrint (" Connecter la SONDE 3 seule ,puis appui sur SW0\r\n");
     Erase_Line(1); __delay_ms(100);
    LCD_putsRom("Connect SONDE #3");
     Erase_Line(2); __delay_ms(100);
    LCD_putsRom("et appui sur SW0");
      __delay_ms(100);
    while(SW0==1)
    {};  
     __delay_ms(20);
    while(SW0==0)    
     {};
    LCD_Cmd(LCD_CLEAR);
    LCD_Cmd(LCD_HOME); //  0x02
    CPrint (" Recup ID# sonde #3 \r\n");
    
    broadcastConvert() ;
    CPrint("  Sensor Type, ID#3 , CRC  :\r\n");
    printSingleAddress();
    
    
      Erase_Line(1); __delay_ms(100);
     LCD_putsRom("ID#3 sensor     ");
     Erase_Line(2); __delay_ms(100);
     sprintf(CRam1,"%8c",OWS_address);
     LCD_Write_Text_At(2,1,CRam1);
     __delay_xSec(3);
    
     CPrint (" appui sur SW0 pour recommencer ou SW4 pour valider\r\n");
    Erase_Line(1);
    LCD_putsRom("valid par   SW4 ");
    Erase_Line(2);
    LCD_putsRom("annule par SW0  ");
      __delay_ms(100);
     while((SW4==1) && (SW0==1))
    { __asm("NOP");
    };
      __delay_ms(20);
    
     if ((SW4==1) && (SW0==0)) 
         goto Encore1;
            
    memcpy(SENSOR_ADDRESS[2],OWS_address,8);
    Flag_Eeprom=Flag_Eeprom+9;   // 90+9=99
    Sauve_Parametres_en_Eeprom() ;
    Relecture_Eeprom(1);
       
    
    
 Suite:
    // si on arrive directement ici, c'est que les ID# sont en EEPROM      
    CPrint (" Connecter les 3 sondes sur RA0 et appui sur SW0 pour continuer\r\n");
     Erase_Line(1); __delay_ms(100);
    LCD_putsRom("Connect.les 3   ");
     Erase_Line(2); __delay_ms(100);
    LCD_putsRom("et appui sur SW0");
    __delay_ms(100);
    while(SW0==1)
      {};
    __delay_ms(20);
    while(SW0==0)
      {};
    LCD_Cmd(LCD_CLEAR);
    LCD_Cmd(LCD_HOME); //  0x02
    // gabarit   1234567890123456
    LCD_putsRom("T.INTER.        ");
    Erase_Line(2); __delay_ms(100);
    LCD_putsRom("T.EXTER.        "); 
    // LCD 4x20 ?
  //  Erase_Line(3); __delay_ms(100);
  //  LCD_putsRom("T.EAU.        ");   
   
    
   i=0;
   CPrint(" 3 Mesures DS18B20 #1 : \r\n");
   do
    {
       broadcastConvert(); 
       F1= getTemperature(SENSOR_ADDRESS[0]);   
       sprintf(txt," %05d  temp#1. EXTERIEUR = %5.3f %c C \t",i,F1,176);
       Print(txt);
       sprintf(CRam1,"%3.2f C",F1);
       LCD_Write_Text_At(1,10,CRam1);
       LCD_Chr_At(1,15,0);   // degre C
     
            
        broadcastConvert(); 
        F1= getTemperature(SENSOR_ADDRESS[1]);   
        sprintf(txt," temp#2. INTERIEUR = %5.3f %c C \t",F1,176);
        Print(txt);
        sprintf(CRam1,"%3.2f C",F1);
        LCD_Write_Text_At(2,10,CRam1);
        LCD_Chr_At(2,15,0);   // degre C
       
        broadcastConvert(); 
        F1= getTemperature(SENSOR_ADDRESS[2]);   
        sprintf(txt," temp#3. EAU  = %5.3f %c C \r\n",F1,176);
        Print(txt);
      // LCD 4x20  
      //  sprintf(CRam1,"%3.2f C",F1);
      //  LCD_Write_Text_At(3,10,CRam1);
      //  LCD_Chr_At(3,15,0);   // degre C
      
       __asm("btg LATA,4");  // LED_Rouge
        i++;
        __delay_xSec(1);
   }
   while (1);  
}

/*   11/04/2023
 (09:22:13.512) 
(09:22:13.512)  Presentation : PIC18F27K42 + OWS sensor DS18B20 sur  PORT RA0)
(09:22:13.539)  Directory :C:\MPLABX_Projects\_18F27K42_OWS_DS18B20_2023.X
(09:22:13.539)  Project :  18F27K42_OWS_DS18B20_2023
(09:22:13.539)  Source :  main.c _2023-0410
(09:22:13.539)  Config  Internal Fosc 64MHz
(09:22:13.539)  Autres :  DS18B20_barker.h  OneWire_barker.h  LCD_4bits_I2C.h UART1_Functions.h
(09:22:13.539)  Eeprom : Eeprom_PIC.h
(09:22:13.566)  RA0=DS18B20,RA4=led,RA3=SQA,RC6=TX UART1
(09:22:13.566)  RC3=SCL,RC4=SDA,BP...BP5 RB0..R5
(09:22:13.566) 
(09:22:13.566)  Compile le Apr 12 2023 a 09:09:09 UTC
(09:22:13.566)  avec version XC8 :  2360 et PACK PIC18F-K_DFP 1.6.125
(09:22:13.566) 
(09:22:13.566)  Init_I2C();
(09:22:13.566)  Test presence devices sur Bus I2C1
(09:22:13.583)  @ decimal #  64  soit @Device 7bits = 0X20  PCF8754 for LCD 2x16cars
(09:22:13.602) 
(09:22:13.602) 
(09:22:13.602)  adresse LCD= 32 soit 0X20 
(09:22:13.602) 
(09:22:13.602)  
(09:22:13.602) Init LCD 2x16 via I2C1 PCF8574
(09:22:13.602)  .123456789A
(09:22:14.638)  Affichage sur LCD 2x16 
(09:22:15.851)  Chargement caracteres Speciaux set #1
(09:22:16.005) DEGREE
(09:22:16.214) EURO  
(09:22:16.424) OHM   
(09:22:16.632) PF    
(09:22:16.841) Droite
(09:22:17.050) Gauche
(09:22:17.258) CANCEL
(09:22:17.468) Check.
(09:22:17.827) 
(09:22:17.827)  
(09:22:17.870)  Config et Init One Wire
(09:22:17.870)   ATTENTION : Forage init EEPROM par programme  pendant le developpement Application,
(09:22:17.870)      pour eviter  une resaisie systematique des ID#  chaque rechargement de programme
(09:22:17.870)   Relecture et visu contenu Eeprom
(09:22:17.896)  Relecture datas eeprom : 
(09:22:17.896)  Flag @ 0x0000 =99
(09:22:17.896)  Code Sonde #0: 0x0010  	0x28,0xFF,0x47,0x15,0x81,0x16,0x03,0x24,
(09:22:18.102)  Code Sonde #1: 0x0020  	0x28,0xAA,0x42,0x65,0x26,0x13,0x02,0xB6,
(09:22:18.291)  Code Sonde #2: 0x0030  	0x28,0xAA,0x56,0xC1,0x38,0x14,0x01,0x40,
(09:22:18.494) 
(09:22:18.494) 
(09:22:20.489)  Parametres en Eeprom restitus OK et transfrs  -> table de travail  SENSOR_ADDRESS[[] 
(09:22:22.847)  Connecter les 3 sondes sur RA0 et appui sur SW0 pour continuer
(09:22:31.730)  3 Mesures DS18B20 #1 : 
(09:22:32.448)  00000  temp#1. EXTERIEUR = 18.187  C 	 temp#2. INTERIEUR = 18.750  C 	 temp#3. EAU  = 18.500  C 
(09:22:35.761)  00001  temp#1. EXTERIEUR = 18.187  C 	 temp#2. INTERIEUR = 18.812  C 	 temp#3. EAU  = 18.500  C 
(09:22:39.076)  00002  temp#1. EXTERIEUR = 18.250  C 	 temp#2. INTERIEUR = 18.812  C 	 temp#3. EAU  = 18.500  C 
(09:22:42.390)  00003  temp#1. EXTERIEUR = 18.250  C 	 temp#2. INTERIEUR = 18.812  C 	
 *  */